﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Newtonsoft.Json;

namespace RESTWebAPI.Models
{
    public class ProductRepository
    {

        #region Properties
        public List<Product> Products { get; set; }
        #endregion Properties

        #region Members
        private ProductsEntities entities;
        #endregion Members

        #region Constructors
        public ProductRepository()
        {
            this.Products = new List<Product>();
            this.Load();
        }
        #endregion

        #region Methods
        private void Load()
        {
            this.entities = new ProductsEntities();
            foreach (ProductEntity p in this.entities.Products)
                this.Products.Add(new Product(p));
        }

        public void Create(Product product)
        {
            List<Product> list = new List<Product>();
            list.Add(product);
            Create(list);
        }

        public void Create(List<Product> products)
        {
            this.Products.AddRange(products);
            // update source
            foreach (Product p in products)
            {
                ProductEntity pe = ProductEntity.CreateProduct(p.ID, p.Name, p.Price, p.Quantity, p.ReleaseDate);
                entities.Products.AddObject(pe);
            }
            entities.SaveChanges();
        }

        public bool Update(int id, Product product)
        {
            List<int> ids = new List<int>();
            ids.Add(id);
            List<Product> products = new List<Product>();
            products.Add(product);
            return Update(ids, products).Count == 0;
        }
        
        public List<Product> Update(List<int> ids, List<Product> products)
        {
            List<Product> updated = this.Products.FindAll(p => ids.Contains<int>(p.ID));
            List<Product> notFound = products.Where<Product>(p => updated.Find(q => q.ID == p.ID) == null).ToList<Product>();

            foreach (Product pOld in updated)
            {
                Product pNew = products.Find(p => p.ID == pOld.ID);
                pOld.Name = pNew.Name;
                pOld.Price = pNew.Price;
                pOld.Quantity = pNew.Quantity;
                pOld.ReleaseDate = pNew.ReleaseDate;
            }

            // update source
            IQueryable<ProductEntity> updatedEnt = entities.Products.Where<ProductEntity>(p => ids.Contains<int>(p.ID));

            foreach (ProductEntity peOld in updatedEnt)
            {
                Product pNew = products.Find(p => p.ID == peOld.ID);
                peOld.Name = pNew.Name;
                peOld.Price = pNew.Price;
                peOld.Quantity = pNew.Quantity;
                peOld.ReleaseDate = pNew.ReleaseDate;
            }
            entities.SaveChanges();
            
            return notFound;
        }

        public Product Delete(int id)
        {
            List<int> list = new List<int>();
            list.Add(id);
            return Delete(list).First<Product>();
        }

        public IEnumerable<Product> Delete(List<int> ids)
        {
            List<Product> productsToRemove = this.Products.FindAll(p => ids.Contains<int>(p.ID));
            this.Products.RemoveAll(p => ids.Contains<int>(p.ID));
            // update source
            var e = entities.Products.Where<ProductEntity>(p => ids.Contains<int>(p.ID));
            foreach (ProductEntity pe in e)
                entities.Products.DeleteObject(pe);
            entities.SaveChanges();

            return productsToRemove;
        }
        #endregion Methods
    }

    public class Product
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public double Price { get; set; }
        public int Quantity { get; set; }
        public DateTime ReleaseDate { get; set; }

        public Product()
        {

        }

        public Product(ProductEntity pe)
        {
            this.ID = pe.ID;
            this.Name = pe.Name;
            this.Price = pe.Price;
            this.Quantity = pe.Quantity;
            this.ReleaseDate = pe.ReleaseDate;
        }
    }
}